(require 'org)
(defvar test-ext-plist nil)
(defvar test-export-arg 0)
(defvar test-start-regexp-ocurrence 1)
(defvar test-log-buffer nil)
(defvar test-temp-buff-1 nil)
(defvar test-temp-buff-2 nil)
(defvar test-number 0)
(defvar test-debug nil
  "Set to non `nil' in order to debug on first error met")
(defvar test-ignore-space t
  "\
* `nil' to keep output as it is 

* `:par' when all white space must bit ignored in output, but a
  skipped lined in remained

* `t' when all white space must be ignored in output")

(defvar test-exporter (symbol-function 'org-export-as-html)
  "Exporter under test")

(defvar test-exporter  'org-export-as-html
  "Exporter under test")

(defvar test-from-start-re "")
(defvar test-upto-end-re "")
(defvar test-delimiter-detecter nil)

(defun test-export (org-code export-code)
  "\
Return nil in case of export OK, error code in case or error.

ORG-CODE is some string, EXPORT-CODE is the expected export
string, `test-exporter' is exported, `test-from-start-re' and
`test-up-to-end-re' are regexp to extract export-code from the
exported code."

  (setq test-number (1+ test-number))
  (let (p1 p2 exported-string)
    (with-current-buffer test-temp-buff-2 (erase-buffer))
    (with-current-buffer test-temp-buff-1
      (erase-buffer)
      (insert org-code)
      (funcall test-exporter
	       test-export-arg nil test-ext-plist test-temp-buff-2
	       (plist-get test-ext-plist :body-only))
      (when (functionp test-delimiter-detecter)
	(funcall test-delimiter-detecter export-code))
      (with-current-buffer test-temp-buff-2
	(goto-char (point-min))
	(let ((found test-start-regexp-ocurrence))
	  (while (and (re-search-forward test-from-start-re nil t) 
		      (> (setq found (1- found)) 0)))
	  (if (= 0 found)
	      (progn
		(setq p1 (match-beginning 0))
		(if (re-search-forward test-upto-end-re nil t)
		    (progn
		      (setq p2 (match-end 0)
			    exported-string (buffer-substring-no-properties p1 p2))
		      (cond
		       ((null test-ignore-space) )
		       ((eq t test-ignore-space) 
			(setq exported-string 
			      (mapconcat 'identity
					 (split-string exported-string "\s-+\\(\ns-*\\)*")
					 " ")))
		       ((eq :par test-ignore-space)
			(setq exported-string (split-string exported-string "\\s-\n\\s-*"))
			(let (prev-lines prev-prev-lines (lines exported-string) line)
			  (while lines
			    (setq line lines
				  lines (cdr lines))
			    (when (string-match "\\`\\s-*\\'" (car line))
			      (setcar line ""))
			    (if (and prev-prev-line
				     (string= (car prev-prev-line) "")
				     (string= (car prev-line) "")
				     (string= (car line) ""))
				(setcdr prev-line lines)
			      (setq prev-prev-line prev-line
				    prev-line line))))
			(setq exported-string (mapconcat 'identity exported-string "\n")
			      exported-string (split-string exported-string "\\s-\\{2,\\}")
			      exported-string (mapconcat 'identity exported-string " ")))
		       (t (error "Unexpected `test-ignore-space' = %S" test-ignore-space)))

		      (if (string= exported-string export-code)
			  nil ; test is OK
			(when test-debug (debug))
			(list
			 "Error unexpected gotten thing"
			 org-code export-code exported-string )))
		  (when test-debug (debug))
		  (list 
		   "Can't find upto-end-re" 
		   org-code export-code  ""
		   )))
	    (when test-debug (debug))
	    (list "Can't find from-start-re"
		  org-code export-code  "")))))))

(defun test-html-delimeter-detecter (export-code)
  (string-match "\\`<\\([a-z]+\\)" export-code)
  (setq test-from-start-re (concat (regexp-quote (match-string 0 export-code))
				   "\\_>")
	test-upto-end-re (regexp-quote (concat "</" (match-string 1 export-code) ">"))))


(defun test-log (x)
  (with-current-buffer test-log-buffer
    (when x
      (insert "\n============================================\n" (car x)
	      ".\nTest number: " (number-to-string test-number)
	      ".\nOrg code:\n =>" (cadr x)
	      ".\nExpected code:\n =>" (caddr x)
	      ".\nGotten code:\n =>" (cadddr x) "." ))))


(defun test-start ()
  (setq test-number 0)
  (with-current-buffer (setq
			test-temp-buff-2 (get-buffer-create " org-tmp2 ")
			test-temp-buff-1 (get-buffer-create " org-tmp1 "))
    (org-mode)
    (erase-buffer))
  (with-current-buffer (setq test-log-buffer (get-buffer-create "*Org Test*"))
    (erase-buffer))  )

(defun test-end ()
  (switch-to-buffer test-log-buffer)
  ;; output content of current buffer for batch mode testing
  (when noninteractive
    (message "%s" (buffer-substring-no-properties (point-min) (point-max)))))



;;;==========================================================================
;;; Starting the export tests
;;;==========================================================================
(test-start)

(plist-put test-ext-plist :body-only t)
(setq test-start-regexp-ocurrence 1
      test-delimiter-detecter (symbol-function 'test-html-delimeter-detecter))

(test-log (test-export "* some test\n[[http:/some/link][xxx]]" 
		       "<a href=\"http:/some/link\">xxx</a>" ))
(test-log (test-export "* some test\n[[http:/some/link][2010-11-01  xxx]]" 
		       "<a href=\"http:/some/link\">2010-11-01 xxx</a>" ))
(test-log (test-export "* some test\n*Bold thing*" 
		       "<b>Bold thing</b>" ))
(test-log (test-export "* some test\nSome^{superscript thing}" 
		       "<sup>superscript thing</sup>" ))


(test-end)